@@ -1,10 +1,8 @@
-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import sys
 
 import numpy as np
 import pytest
+
 xr = pytest.importorskip('xarray')  # noqa
 
 from cf2cdm import cfcoords
@@ -12,8 +10,8 @@ from cf2cdm import cfcoords
 
 @pytest.fixture
 def da1():
-    latitude = [0.5, 0.]
-    longitude = [10., 10.5]
+    latitude = [0.5, 0.0]
+    longitude = [10.0, 10.5]
     time = ['2017-12-01T00:00:00', '2017-12-01T12:00:00', '2017-12-02T00:00:00']
     level = [950, 500]
     data = xr.DataArray(
@@ -21,17 +19,21 @@ def da1():
         coords=[
             ('lat', latitude, {'units': 'degrees_north'}),
             ('lon', longitude, {'units': 'degrees_east'}),
-            ('ref_time', np.array(time, dtype=np.datetime64),
-                {'standard_name': 'forecast_reference_time'}),
+            (
+                'ref_time',
+                np.array(time, dtype=np.datetime64),
+                {'standard_name': 'forecast_reference_time'},
+            ),
             ('level', np.array(level), {'units': 'hPa'}),
-        ])
+        ],
+    )
     return data
 
 
 @pytest.fixture
 def da2():
-    latitude = [0.5, 0.]
-    longitude = [10., 10.5]
+    latitude = [0.5, 0.0]
+    longitude = [10.0, 10.5]
     time = ['2017-12-01T00:00:00', '2017-12-01T12:00:00', '2017-12-02T00:00:00']
     level = [950, 500]
     data = xr.DataArray(
@@ -41,14 +43,15 @@ def da2():
             ('lon', longitude, {'units': 'degrees_east'}),
             ('time', np.array(time, dtype=np.datetime64)),
             ('level', np.array(level), {'units': 'hPa'}),
-        ])
+        ],
+    )
     return data
 
 
 @pytest.fixture
 def da3():
-    latitude = [0.5, 0.]
-    longitude = [10., 10.5]
+    latitude = [0.5, 0.0]
+    longitude = [10.0, 10.5]
     step = [0, 24, 48]
     time = ['2017-12-01T00:00:00', '2017-12-01T12:00:00']
     level = [950, 500]
@@ -58,10 +61,14 @@ def da3():
             ('lat', latitude, {'units': 'degrees_north'}),
             ('lon', longitude, {'units': 'degrees_east'}),
             ('step', np.array(step, dtype=np.timedelta64), {'standard_name': 'forecast_period'}),
-            ('ref_time', np.array(time, dtype=np.datetime64),
-                {'standard_name': 'forecast_reference_time'}),
+            (
+                'ref_time',
+                np.array(time, dtype=np.datetime64),
+                {'standard_name': 'forecast_reference_time'},
+            ),
             ('time', np.array(level), {'units': 'hPa'}),
-        ])
+        ],
+    )
 
     return data
 
@@ -133,8 +140,19 @@ def test_translate_coords(da1, da2, da3):
 
 @pytest.mark.skipif(sys.version_info < (3, 6), reason="test needs stable dict's")
 def test_translate_coords_errors(da3):
+    cfcoords.translate_coords(da3)
+    cfcoords.translate_coords(da3, errors='ignore')
     with pytest.raises(RuntimeError):
-        cfcoords.translate_coords(da3)
+        cfcoords.translate_coords(da3, errors='raise')
+
+    DATA_MODEL = {'config': {'preferred_time_dimension': 'valid_time'}}
+    cfcoords.translate_coords(da3, DATA_MODEL)
+
+    da3_fail = da3.drop('time')
+    cfcoords.translate_coords(da3_fail, DATA_MODEL)
+    cfcoords.translate_coords(da3_fail, DATA_MODEL, errors='ignore')
+    with pytest.raises(RuntimeError):
+        cfcoords.translate_coords(da3_fail, DATA_MODEL, errors='raise')
 
 
 def test_ensure_valid_time(da1, da3):
@@ -152,8 +170,6 @@ def test_ensure_valid_time(da1, da3):
 
     res1 = cfcoords.ensure_valid_time(da3.squeeze())
     assert 'valid_time' in res1.coords
-    coords1 = res1.coords
-    assert coords1['valid_time'].shape == (coords1['ref_time'].size, coords1['step'].size)
 
     with pytest.raises(ValueError):
         cfcoords.ensure_valid_time(da3.mean(dim='ref_time').squeeze())
